'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setupAgenda = setupAgenda;
exports.cullBodies = cullBodies;

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _config = require('./config');

var _model = require('./model');

var _winston = require('winston');

var _winston2 = _interopRequireDefault(_winston);

var _contentChunk = require('./contentChunk');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_config.config.bodyCull = _config.config.get('bodyCull');

function setupAgenda(agenda) {
  if (_config.config.bodyCull == null) {
    return;
  }
  agenda.define('transaction body culling', async (job, done) => {
    try {
      await cullBodies();
      done();
    } catch (err) {
      done(err);
    }
  });
  agenda.every(`${_config.config.bodyCull.pollPeriodMins} minutes`, `transaction body culling`);
}

async function cullBodies() {
  const channels = await _model.ChannelModel.find({ maxBodyAgeDays: { $exists: true } });
  await Promise.all(channels.map(channel => clearTransactions(channel)));
}

async function clearTransactions(channel) {
  const { maxBodyAgeDays, lastBodyCleared } = channel;
  const maxAge = (0, _moment2.default)().subtract(maxBodyAgeDays, 'd').toDate();
  const query = {
    channelID: channel._id,
    'request.timestamp': {
      $lte: maxAge
    }
  };

  if (lastBodyCleared != null) {
    query['request.timestamp'].$gte = lastBodyCleared;
  }

  // construct promises array for removing transaction bodies
  const transactionsToCullBody = await _model.TransactionModel.find(query, {
    'request.bodyId': 1,
    'response.bodyId': 1,
    'orchestrations.response.bodyId': 1,
    'orchestrations.request.bodyId': 1,
    'routes.response.bodyId': 1,
    'routes.request.bodyId': 1,
    'routes.orchestrations.request.bodyId': 1,
    'routes.orchestrations.response.bodyId': 1
  });
  let removeBodyPromises = [];
  for (let tx of transactionsToCullBody) {
    removeBodyPromises = removeBodyPromises.concat((await (0, _contentChunk.promisesToRemoveAllTransactionBodies)(tx)));
  }

  channel.lastBodyCleared = Date.now();
  channel.updatedBy = { name: 'Cron' };
  await channel.save();
  const updateResp = await _model.TransactionModel.updateMany(query, {
    $unset: {
      "request.bodyId": "",
      "response.bodyId": "",
      "orchestrations.$[].request.bodyId": "",
      "orchestrations.$[].response.bodyId": "",
      "routes.$[].request.bodyId": "",
      "routes.$[].response.bodyId": "",
      "routes.$[].orchestrations.$[].request.bodyId": "",
      "routes.$[].orchestrations.$[].response.bodyId": ""
    }
  });
  if (updateResp.nModified > 0) {
    _winston2.default.info(`Culled ${updateResp.nModified} transaction bodies for channel ${channel.name}`);
  }

  // execute the promises to remove all relevant bodies
  await Promise.all(removeBodyPromises.map(promiseFn => promiseFn()));
}
//# sourceMappingURL=bodyCull.js.map