'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setupApp = setupApp;
exports.rerunApp = rerunApp;
exports.tcpApp = tcpApp;
exports.pollingApp = pollingApp;

var _koa = require('koa');

var _koa2 = _interopRequireDefault(_koa);

var _rawBody = require('raw-body');

var _rawBody2 = _interopRequireDefault(_rawBody);

var _koaCompress = require('koa-compress');

var _koaCompress2 = _interopRequireDefault(_koaCompress);

var _zlib = require('zlib');

var _winston = require('winston');

var _winston2 = _interopRequireDefault(_winston);

var _router = require('./middleware/router');

var router = _interopRequireWildcard(_router);

var _messageStore = require('./middleware/messageStore');

var messageStore = _interopRequireWildcard(_messageStore);

var _basicAuthentication = require('./middleware/basicAuthentication');

var basicAuthentication = _interopRequireWildcard(_basicAuthentication);

var _tlsAuthentication = require('./middleware/tlsAuthentication');

var tlsAuthentication = _interopRequireWildcard(_tlsAuthentication);

var _rerunBypassAuthentication = require('./middleware/rerunBypassAuthentication');

var rerunBypassAuthentication = _interopRequireWildcard(_rerunBypassAuthentication);

var _rerunBypassAuthorisation = require('./middleware/rerunBypassAuthorisation');

var rerunBypassAuthorisation = _interopRequireWildcard(_rerunBypassAuthorisation);

var _rerunUpdateTransactionTask = require('./middleware/rerunUpdateTransactionTask');

var rerunUpdateTransactionTask = _interopRequireWildcard(_rerunUpdateTransactionTask);

var _tcpBypassAuthentication = require('./middleware/tcpBypassAuthentication');

var tcpBypassAuthentication = _interopRequireWildcard(_tcpBypassAuthentication);

var _retrieveTCPTransaction = require('./middleware/retrieveTCPTransaction');

var retrieveTCPTransaction = _interopRequireWildcard(_retrieveTCPTransaction);

var _requestMatching = require('./middleware/requestMatching');

var requestMatching = _interopRequireWildcard(_requestMatching);

var _authorisation = require('./middleware/authorisation');

var authorisation = _interopRequireWildcard(_authorisation);

var _pollingBypassAuthorisation = require('./middleware/pollingBypassAuthorisation');

var pollingBypassAuthorisation = _interopRequireWildcard(_pollingBypassAuthorisation);

var _pollingBypassAuthentication = require('./middleware/pollingBypassAuthentication');

var pollingBypassAuthentication = _interopRequireWildcard(_pollingBypassAuthentication);

var _rawBodyReader = require('./middleware/rawBodyReader');

var rawBodyReader = _interopRequireWildcard(_rawBodyReader);

var _events = require('./middleware/events');

var events = _interopRequireWildcard(_events);

var _proxy = require('./middleware/proxy');

var proxy = _interopRequireWildcard(_proxy);

var _config = require('./config');

var _tls = require('tls');

var _stream = require('stream');

var _util = require('util');

var _contentChunk = require('./contentChunk');

var _mongoose = require('mongoose');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// TODO: OHM-696 uncomment the line below
//import * as rewrite from './middleware/rewriteUrls'
_config.config.authentication = _config.config.get('authentication');

// Primary app

function setupApp(done) {
  const app = new _koa2.default();

  // Basic authentication middleware
  if (_config.config.authentication.enableBasicAuthentication) {
    app.use(basicAuthentication.koaMiddleware);
  }

  // TLS authentication middleware
  if (_config.config.authentication.enableMutualTLSAuthentication) {
    app.use(tlsAuthentication.koaMiddleware);
  }

  // Request Matching middleware
  app.use(requestMatching.koaMiddleware);

  // Authorisation middleware
  app.use(authorisation.koaMiddleware);

  app.use(rawBodyReader.koaMiddleware);

  // Compress response on exit
  app.use((0, _koaCompress2.default)({
    threshold: 8,
    flush: _zlib.Z_SYNC_FLUSH
  }));

  // Proxy
  app.use(proxy.koaMiddleware);

  // Persist message middleware
  //app.use(messageStore.koaMiddleware)

  // URL rewriting middleware
  // TODO: OHM-696 uncomment the code below when url rewriting is back in support
  // app.use(rewrite.koaMiddleware)

  // Events
  app.use(events.koaMiddleware);

  // Call router
  app.use(router.koaMiddleware);

  return done(app);
}

// Rerun app that bypasses auth
function rerunApp(done) {
  const app = new _koa2.default();

  // Rerun bypass authentication middlware
  app.use(rerunBypassAuthentication.koaMiddleware);

  // Rerun bypass authorisation middlware
  app.use(rerunBypassAuthorisation.koaMiddleware);

  // Persist message middleware
  //app.use(messageStore.koaMiddleware)

  // Authorisation middleware
  app.use(authorisation.koaMiddleware);

  app.use(rawBodyReader.koaMiddleware);

  // Update original transaction with rerunned transaction ID
  app.use(rerunUpdateTransactionTask.koaMiddleware);

  // Events
  app.use(events.koaMiddleware);

  // Call router
  app.use(router.koaMiddleware);

  return done(app);
}

// App for TCP/TLS sockets
function tcpApp(done) {
  const app = new _koa2.default();

  app.use(rawBodyReader.koaMiddleware);
  app.use(retrieveTCPTransaction.koaMiddleware);

  // TCP bypass authentication middlware
  app.use(tcpBypassAuthentication.koaMiddleware);

  // Proxy
  app.use(proxy.koaMiddleware);

  // Persist message middleware
  app.use(messageStore.koaMiddleware);

  // Events
  app.use(events.koaMiddleware);

  // Call router
  app.use(router.koaMiddleware);

  return done(app);
}

// App used by scheduled polling
function pollingApp(done) {
  const app = new _koa2.default();

  app.use(rawBodyReader.koaMiddleware);

  // Polling bypass authentication middlware
  app.use(pollingBypassAuthentication.koaMiddleware);

  // Polling bypass authorisation middleware
  app.use(pollingBypassAuthorisation.koaMiddleware);

  // Persist message middleware
  app.use(messageStore.koaMiddleware);

  // Events
  app.use(events.koaMiddleware);

  // Call router
  app.use(router.koaMiddleware);

  return done(app);
}
//# sourceMappingURL=koaMiddleware.js.map