'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.koaMiddleware = koaMiddleware;

var _koa = require('koa');

var _koa2 = _interopRequireDefault(_koa);

var _rawBody = require('raw-body');

var _rawBody2 = _interopRequireDefault(_rawBody);

var _koaCompress = require('koa-compress');

var _koaCompress2 = _interopRequireDefault(_koaCompress);

var _zlib = require('zlib');

var _winston = require('winston');

var _winston2 = _interopRequireDefault(_winston);

var _messageStore = require('./messageStore');

var messageStore = _interopRequireWildcard(_messageStore);

var _config = require('../config');

var _stream = require('stream');

var _util = require('util');

var _contentChunk = require('../contentChunk');

var _mongoose = require('mongoose');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_config.config.authentication = _config.config.get('authentication');
// TODO: OHM-696 uncomment the line below
//import * as rewrite from './middleware/rewriteUrls'


let bucket;

async function rawBodyReader(ctx) {
  let counter = 0;
  let size = 0;

  if (!bucket) {
    bucket = (0, _contentChunk.getGridFSBucket)();
  }

  ctx.state.downstream = new _stream.Readable();
  ctx.state.downstream._read = () => {};

  let gridFsStream;

  /*
   * Only transactions that were requested to be rerun should have this 
   * custom header (the GridFS fileId of the body for this transaction)
   */
  const bodyId = ctx.request.headers['x-body-id'];
  const requestHasBody = ['POST', 'PUT', 'PATCH'].includes(ctx.req.method) && bodyId == null;

  if (['POST', 'PUT', 'PATCH'].includes(ctx.req.method)) {
    if (requestHasBody) {
      /*
      *   Request has a body, so stream it into GridFs
      */
      gridFsStream = bucket.openUploadStream();

      ctx.requestTimestamp = new Date();

      // Get the GridFS file object that was created
      ctx.request.bodyId = gridFsStream.id;

      // Create the transaction for Request (started receiving)
      // Side effect: Updates the Koa ctx with the transactionId
      ctx.state.requestPromise = messageStore.initiateRequest(ctx);

      gridFsStream.on('error', err => {
        _winston2.default.error(`Couldn't stream request into GridFS for fileId: ${ctx.request.bodyId} - ${err}`);
      });
    } else {
      /*
      *   Request has a bodyId (it's a rerun), so stream the body from GridFs 
      *      and send it downstream
      */
      const fileId = new _mongoose.Types.ObjectId(bodyId);
      gridFsStream = bucket.openDownloadStream(fileId);

      ctx.request.bodyId = fileId;
      ctx.state.requestPromise = messageStore.initiateRequest(ctx);

      gridFsStream.on('data', chunk => {
        ctx.req.push(chunk);
      }).on('end', () => {
        _winston2.default.info(`** END OF INPUT GRIDFS STREAM **`);
        ctx.req.push(null);
      }).on('error', err => {
        _winston2.default.error(`Cannot stream request body from GridFS for fileId: ${bodyId} - ${err}`);
      });
    }
  } else {
    /*
     *  GET and DELETE come in here to persist the intial request transaction
     */
    ctx.state.requestPromise = messageStore.initiateRequest(ctx);
  }

  ctx.req.on('data', chunk => {
    counter++;
    size += chunk.toString().length;
    _winston2.default.info(`Read request CHUNK # ${counter} [ Total size ${size}]`);

    // Write chunk to GridFS & downstream
    if (requestHasBody) {
      gridFsStream.write(chunk);
    }
    ctx.state.downstream.push(chunk);
  }).on('end', () => {
    _winston2.default.info(`** END OF INPUT STREAM **`);

    // Close streams to gridFS and downstream
    if (requestHasBody) {
      gridFsStream.end();
    }
    ctx.state.downstream.push(null);

    // Update the transaction for Request (finished receiving)
    // Only update after `messageStore.initiateRequest` has completed
    ctx.state.requestPromise.then(() => {
      messageStore.completeRequest(ctx, () => {});
    });
  }).on('error', err => {
    _winston2.default.error(`Couldn't read request stream from socket: ${err}`);
  });
}

/*
 * Koa middleware for streaming to GridFS and streaming routing
 */
async function koaMiddleware(ctx, next) {
  rawBodyReader(ctx);
  await next();
}
//# sourceMappingURL=rawBodyReader.js.map